"""
base forms app
"""
from installations import models as installations_models
from django import forms
from utils import forms as utils_forms
from utils import fields as utils_fields
from utils import lists as utils_lists
from utils import widgets as utils_widgets
from utils import sheets
from utils.versioning import versionedforms
from utils.versioning.models import WorkVersion
from django.utils.translation import ugettext_lazy as _

class ScenarioFilterForm(utils_lists.FilterForm):
    """Custom filterform for the scenario list"""
    name = forms.CharField(max_length=100, required=False, label=_('Scenario name'))
    type = forms.ChoiceField(choices=(), required=False, label = _('Category'))
    is_ok = forms.ChoiceField(choices=(), required=False, label = _('Evaluation'))
    class Meta:
##        model = app_models.DegradationProtectionLayer
        attrs = {}
    def get_scenario_type_choices(self, version):
        choices = [('', '---------')]
        choices.extend([(a,a) for a in self.Meta.model.objects.filter(suggestionlist__isnull=True
            ).exclude(type=u''
            ).order_by('type'
            ).get_versionedobjects(version
            ).values_list('type', flat=True
            ).distinct()])
        return choices
    def get_scenario_is_ok_choices(self):
        from analyses.analysisbase.models import IS_OK_CHOICES
        choices = [('', '---------')]
        choices.extend(IS_OK_CHOICES)
        return choices
    def __init__(self, version=None, *args, **kwargs):
        super(ScenarioFilterForm, self).__init__(*args, **kwargs)
        self.fields['type'].choices = self.get_scenario_type_choices(version)
        self.fields['is_ok'].choices = self.get_scenario_is_ok_choices()
        self.fields['name'].widget = utils_widgets.JQueryAutoCompleteTextInput(
                namedsource = self.Meta.model.get_name_lookup_url(), attrs={'size':50}
            )

class ScenarioForm(versionedforms.VersionedForm):
    type = forms.CharField(max_length=100, required=False, label=_('Category'))
    class Meta:
##        model = app_models.DegradationScenario
        attrs = {
            'name': {'size':75},
            'type': {'size':75},
            'description': {'rows':10,'cols':65},
        }
        widgets = {
            'description': utils_widgets.MarkItUpWidget()
        }
        fields = ('name', 'type', 'description')
    def __init__(self, version=None, *args, **kwargs):
        super(ScenarioForm, self).__init__(*args, **kwargs)
        self.fields['type'].widget = utils_widgets.JQueryAutoCompleteTextInput(
                namedsource = self.Meta.model.get_type_lookup_url(),
                attrs = {'size': 75},
            )

class ScenarioAddForm(versionedforms.VersionedForm):
    consequence = forms.CharField(max_length=100, required=True, label=_('Consequence'), widget=forms.TextInput(attrs={'size':75}))

    class Meta:
##        model = app_models.DegradationScenario
        attrs = {
            'name': {'size':75},
            'type': {'size':75},
            'description': {'rows':10,'cols':65},
        }
        widgets = {
            'description': utils_widgets.MarkItUpWidget()
        }
        fields = ('name', 'type', 'description', 'consequence')
    def __init__(self, version=None, *args, **kwargs):
        super(ScenarioAddForm, self).__init__(*args, **kwargs)
        self.fields['consequence'].label = self.Meta.model.appconfig.consequence_label
        self.fields['type'].widget = utils_widgets.JQueryAutoCompleteTextInput(
                namedsource = self.Meta.model.get_type_lookup_url(),
                attrs = {'size': 75},
            )

class ScenarioSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
##        model = app_models.DegradationScenario
        fields = ('name', 'type', 'description')

class ScenarioEvaluationForm(versionedforms.VersionedForm):
    class Meta:
##        model = app_models.DegradationScenario
        attrs = {
            'evaluation': {'rows':10,'cols':65},
        }
        widgets = {
            'evaluation': utils_widgets.MarkItUpWidget()
        }
        fields = ('evaluation', 'is_ok')

class ScenarioEvaluationSheet(versionedforms.VersionedSheet):
    evaluation = utils_fields.MarkdownField(data = 'evaluation', name='evaluation', label=_('Evaluation'))
    class Meta:
##        model = app_models.DegradationScenario
        fields = ('evaluation', 'is_ok')

class ScenarioCompareSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    evaluation = utils_fields.MarkdownField(data = 'evaluation', name='evaluation', label=_('Evaluation'))
    class Meta:
##        model = app_models.DegradationScenario
        fields = ('name', 'type', 'description', 'evaluation', 'is_ok', 'scope', 'suggestionlist')
    def __init__(self, instance, *args, **kwargs):
        super(ScenarioCompareSheet, self).__init__(instance, *args, **kwargs)
        if instance.suggestionlist:
            del self.fields['scope']
        else:
            del self.fields['suggestionlist']

class ScenarioReportSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    evaluation = utils_fields.MarkdownField(data = 'evaluation', name='evaluation', label=_('Evaluation'))
    class Meta:
##        model = app_models.DegradationScenario
        fields = ('type', 'description', 'evaluation', 'is_ok', 'scope', 'suggestionlist')
    def __init__(self, instance, *args, **kwargs):
        super(ScenarioReportSheet, self).__init__(instance, *args, **kwargs)
        if instance.suggestionlist:
            del self.fields['scope']
        else:
            del self.fields['suggestionlist']

class ScenarioMoveForm(forms.Form):
    target = forms.ModelChoiceField(queryset=installations_models.Scope.objects.none(), required=True, label = _('Target'))

class EventForm(versionedforms.VersionedForm):
    class Meta:
##        model = app_models.DegradationEvent
        fields = ('name', 'description',)
        attrs = {
            'name': {'size':75},
            'description': {'rows':10,'cols':65},
        }
        widgets = {
            'description': utils_widgets.MarkItUpWidget()
        }

class EventSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
##        model = app_models.DegradationEvent
        pass
    def __init__(self, report=False, *args, **kwargs):
        super(EventSheet, self).__init__(report=report, *args, **kwargs)
        if report:
            del self.fields['name']

class ProtectionLayerFilterForm(utils_lists.FilterForm):
    """Custom filterform for the protection layers list"""
    name = forms.CharField(max_length=100, required=False, label=_('Protection layer name'))
    linked = forms.ChoiceField(required=False, label=_('Linked'),choices = ((u'', u'---------'), (u'1', _('Yes')), (u'0', _('No'))))

    class Meta:
##        model = app_models.DegradationProtectionLayer
        attrs = {
            'name': {'size': 50},
        }

class ProtectionLayerForm(versionedforms.VersionedForm):
    class Meta:
##        model = app_models.DegradationProtectionLayer
##        me_model = app_models.DegradationMeasure
        attrs = {
            'name': {'size':75},
            'description': {'rows':10,'cols':65},
            'effectivity': {'rows':5,'cols':65},
            'reliability': {'rows':5,'cols':65},
            'risks': {'rows':5,'cols':65},
        }
        widgets = {
            'description': utils_widgets.MarkItUpWidget(),
            'effectivity': utils_widgets.MarkItUpWidget(),
            'reliability': utils_widgets.MarkItUpWidget(),
            'risks': utils_widgets.MarkItUpWidget(),
        }
        exclude = ('SL', 'measures')

class ProtectionLayerSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
##        model = app_models.DegradationProtectionLayer
        exclude = ('SL',)
    def __init__(self, report=False, *args, **kwargs):
        super(ProtectionLayerSheet, self).__init__(report=report, *args, **kwargs)
        if report:
            del self.fields['name']

class MeasureFilterForm(utils_lists.FilterForm):
    """Custom filterform for the measures list"""
    name = forms.CharField(max_length=100, required=False, label=_('Measure name'))
    tag = forms.CharField(max_length=100, required=False, label=_('Tag'))
    type = forms.ChoiceField(choices=(), required=False, label = _('Category'))
    linked = forms.ChoiceField(required=False, label=_('Linked'),choices = ((u'', u'---------'), (u'1', _('Yes')), (u'0', _('No'))))

    class Meta:
##        model = app_models.DegradationMeasure
        attrs = {
            'tag': {'size': 20},
        }
    def get_measure_type_choices(self, version):
        choices = [('', '---------')]
        choices.extend([(a,a) for a in self.Meta.model.objects.filter(SL=False
            ).exclude(type=u''
            ).order_by('type'
            ).get_versionedobjects(version
            ).values_list('type', flat=True
            ).distinct()])
        return choices
    def __init__(self, version=None, *args, **kwargs):
        super(MeasureFilterForm, self).__init__(*args, **kwargs)
        self.fields['type'].choices = self.get_measure_type_choices(version)
        self.fields['name'].widget = utils_widgets.JQueryAutoCompleteTextInput(
                namedsource = self.Meta.model.get_name_lookup_url(), attrs={'size':50}
            )

class MeasureForm(versionedforms.VersionedForm):
    type = forms.CharField(max_length=100, required=False, label=_('Category'))
    class Meta:
##        model = app_models.DegradationMeasure
        attrs = {
            'name': {'size':75},
            'tag': {'size':40},
            'type': {'size':40},
            'description': {'rows':10,'cols':65},
        }
        widgets = {
            'description': utils_widgets.MarkItUpWidget()
        }
        fields = ('name', 'tag', 'type', 'description')
    def __init__(self, *args, **kwargs):
        super(MeasureForm, self).__init__(*args, **kwargs)
        self.fields['type'].widget = utils_widgets.JQueryAutoCompleteTextInput(
                namedsource = self.Meta.model.get_type_lookup_url()
            )

class MeasureSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
##        model = app_models.DegradationMeasure
        fields = ('name', 'tag', 'type', 'description')

class MeasureReportSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
##        model = app_models.DegradationMeasure
        fields = ('tag', 'type', 'description')

class LinkMeasureForm(forms.Form):
    measure = forms.ChoiceField(choices = (), required=False, label=_('Measure'), widget = utils_widgets.JQueryCombobox)
    class Meta:
##        model = app_models.DegradationMeasure
        pass
    def get_measure_choices(self, SL):
        choices = [('', '---------')]
        choices.extend([(item.id, item.__unicode__()) for item in self.Meta.model.objects.filter(SL=SL).get_versionedobjects(version=WorkVersion).order_by('name')])
        return choices

    def __init__(self, SL, *args, **kwargs):
        super(LinkMeasureForm, self).__init__(*args, **kwargs)
        self.fields['measure'].choices = self.get_measure_choices(SL)

class ReplaceMeasureForm(forms.Form):
    measure = forms.ChoiceField(choices = (), required=True, label=_('Measure'))
    class Meta:
##        model = app_models.DegradationMeasure
        pass
    def get_measure_choices(self, SL):
        queryset = self.Meta.model.objects.filter(SL=SL).get_versionedobjects(version=WorkVersion).order_by('type', 'name')
        choices = []
        inner = []
        type = None
        for item in queryset:
            itype = item.type or u'---'
            if type is not None and itype != type:
                if type == u'---':
                    choices.extend(inner)
                else:
                    choices.append((type, inner))
                inner = []
                type = itype
            if type is None:
                type = itype
            inner.append((item.id, item.__unicode__()))
        if type == u'---':
            choices.extend(inner)
        else:
            choices.append((type, inner))
        return choices
    def __init__(self, SL, *args, **kwargs):
        super(ReplaceMeasureForm, self).__init__(*args, **kwargs)
        self.fields['measure'].choices = self.get_measure_choices(SL)

class ProtectionLayer_MeasureSheet(versionedforms.VersionedSheet):
    class Meta:
##        model = app_models.DegradationProtectionLayer_Measure
        pass

class AttentionPointFilterForm(utils_lists.FilterForm):
    """Custom filterform for the protection layers list"""
    name = forms.CharField(max_length=100, required=False, label=_('Attention point'))
    type = forms.ChoiceField(choices=(), required=False, label = _('Category'))
    class Meta:
##        model = app_models.DegradationAttentionPoint
        attrs = {}
    def get_attentionpiont_type_choices(self, version):
        choices = [('', '---------')]
        choices.extend([(a,a) for a in self.Meta.model.objects.filter(SL=False
            ).exclude(type=u''
            ).order_by('type'
            ).get_versionedobjects(version
            ).values_list('type', flat=True
            ).distinct()])
        return choices
    def __init__(self, version=None, *args, **kwargs):
        super(AttentionPointFilterForm, self).__init__(*args, **kwargs)
        self.fields['type'].choices = self.get_attentionpiont_type_choices(version)
        self.fields['name'].widget = utils_widgets.JQueryAutoCompleteTextInput(
                namedsource = self.Meta.model.get_name_lookup_url(), attrs={'size':50}
        )

class AttentionPointForm(versionedforms.VersionedForm):
    type = forms.CharField(max_length=100, required=False, label=_('Category'))
    class Meta:
##        model = app_models.DegradationAttentionPoint
        attrs = {
            'name': {'size':75},
            'type': {'size':40},
            'description': {'rows':10,'cols':65},
        }
        widgets = {
            'description': utils_widgets.MarkItUpWidget()
        }
        fields = ('name', 'type', 'description',)

    def __init__(self, *args, **kwargs):
        super(AttentionPointForm, self).__init__(*args, **kwargs)
        self.fields['type'].widget = utils_widgets.JQueryAutoCompleteTextInput(
                namedsource = self.Meta.model.get_type_lookup_url()
            )

class AttentionPointSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
        pass
##        model = app_models.DegradationAttentionPoint
        fields = ('name', 'type', 'description', 'protectionlayer')
        links = ('protectionlayer',)

class AttentionPointReportSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
##        model = app_models.DegradationAttentionPoint
        fields = ('type', 'description', 'measure')

class AttentionPointSubReportSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
##        model = app_models.DegradationAttentionPoint
        fields = ('type', 'description',)

class SuggestionlistForm(versionedforms.VersionedForm):
    class Meta:
##        model = app_models.DegradationSuggestionlist
        attrs = {
            'name': {'size':75},
            'description': {'rows':10,'cols':65},
        }
        widgets = {
            'description': utils_widgets.MarkItUpWidget()
        }
        exclude = ('system',)

class SuggestionlistSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
##        model = app_models.DegradationSuggestionlist
        exclude = ('system',)

class SuggestionListScenarioForm(versionedforms.VersionedForm):
    """
    Form for single select
    Instance is the scenario from the SL to be copied
    Added 'id' field to avoid getting all fields
    """
    select = forms.BooleanField(required=False, widget=forms.CheckboxInput)
    class Meta:
##        model = app_models.DegradationScenario
        fields = ('id',)

class SuggestionListInlineFormset(versionedforms.VersionedInlineFormSet):
    """
    Formset with SL select forms
    """
    extra = 0
##    form = SuggestionListScenarioForm
##    model = app_models.DegradationScenario
    targetfield = 'scope'
    parent = 'suggestionlist'

    def __init__(self, target=None, **kwargs):
        """
        Pass in target scope so save can copy the selected scenarios
        """
        self.target = target
        super(SuggestionListInlineFormset, self).__init__(**kwargs)

    def save(self, user):
        counter = 0
        for form in self.forms:
            if form.cleaned_data['select']:
                counter += 1
                form.instance.copy(user, suffix=_('[SL]'), **{self.targetfield:self.target})
        return counter


class ApSuggestionlistReportSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
##        model = app_models.DegradationAPSuggestionlist
        exclude = ('system', 'name')

class AllSuggestionlistsReportForm(forms.Form):
    def __init__(self, appconfig, user, *args, **kwargs):
        from analyses.analysisbase.models import AllSuggestionlists
        super(AllSuggestionlistsReportForm, self).__init__(*args, **kwargs)
        fields = utils_forms.treeify(AllSuggestionlists.get_report_formfields(appconfig, user))
        for k in fields:
            self.fields[k['name']] = k['field']
