"""
collect tasks for installations

"""
from django.contrib.contenttypes.models import ContentType

from analyses.main import analysesmodules
from analyses.analysisbase.models import EVENT_ID, PROTECTIONLAYER_ID
from installations import models as installations_models
from tasks import models as app_models
from utils.versioning import models as versioning_models
import datetime

def chunks(l, n):
    """ Yield successive n-sized chunks from l.
    """
    for i in xrange(0, len(l), n):
        yield l[i:i+n]

def get_tasks_for_installation(sectiontreeelement, Qobjs, params, user):
    version = getattr(sectiontreeelement, '_version', sectiontreeelement.version) or versioning_models.WorkVersion
    tasks = []

    # tasks for sections
    stes = sectiontreeelement.get_descendants(include_self=True).get_versionedobjects(version=version).select_related_versioned()
    sections = [element.section_id for element in stes]
    if sections:
        object_type = ContentType.objects.get_for_model(installations_models.Section)
        if Qobjs or params:
            qs = app_models.Task.objects.filter(*Qobjs, **params)
        else:
            qs = app_models.Task.objects.all()
        for sections_chunk  in chunks(sections, 100):
            tasks.extend(list(qs.filter(content_type__pk=object_type.id,
                object_id__in=sections_chunk).get_versionedobjects(version=version)))

        scopes = installations_models.Scope_Section.objects.filter(section__in=sections
                ).get_versionedobjects(version).values_list('scope_id', flat=True)
        for module in analysesmodules.get_items_sorted():
            models = module['appconfig'].get_models()
            scmodel = models['scenario']
            if scmodel.may_display(user):
                scenarios = scmodel.objects.filter(scope__in=scopes
                    ).get_versionedobjects(version=version
                    ).select_related_versioned('scope')
                tasks.extend(get_tasks_for_scenarios(qs, version, scenarios, Qobjs, params, models))

    # now let's sort
    tasks = sorted(tasks, key=lambda t: t.duedate or datetime.date(1980,1,1))

    return tasks


def get_tasks_for_scenarios(qs, version, scenarios, Qobjs, params, models):
    tasks = []
    if scenarios:
        scenario_type = ContentType.objects.get_for_model(models['scenario'])
        event_type = ContentType.objects.get_for_model(models['event'])
        protectionlayer_type = ContentType.objects.get_for_model(models['protectionlayer'])
        measure_type = ContentType.objects.get_for_model(models['measure'])
        attentionpoint_type = ContentType.objects.get_for_model(models['attentionpoint'])

        sc = [item.v_id for item in scenarios]
        for sc_chunk in chunks(sc, 100):
            # add tasks for scenarios
            tasks.extend(list(qs.filter(content_type__pk=scenario_type.id,
                object_id__in=sc_chunk).get_versionedobjects(version=version)))

            # causes
            items = models['treeelement'].objects.filter(scenario__in=sc_chunk
                ).filter(elementtype=EVENT_ID
                ).get_versionedobjects(version=version
                ).values_list('object_id', flat=True)
            if items:
                for items_chunk in chunks(items, 100):
                    # tasks for causes
                    tasks.extend(list(qs.filter(content_type__pk=event_type.id,
                        object_id__in=items_chunk).get_versionedobjects(version=version)))

            # protectionlayers
            items = models['treeelement'].objects.filter(scenario__in=sc_chunk
                ).filter(elementtype=PROTECTIONLAYER_ID
                ).get_versionedobjects(version=version
                ).values_list('object_id', flat=True)
            if items:
                for items_chunk in chunks(items, 100):
                    # tasks for protectionlayers
                    tasks.extend(list(qs.filter(content_type__pk=protectionlayer_type.id,
                        object_id__in=items_chunk).get_versionedobjects(version=version)))
                    m_ids = models['protectionlayer_measure'].objects.filter(protectionlayer__in=items_chunk
                        ).get_versionedobjects(version=version
                        ).values_list('measure_id', flat=True)
                    if m_ids:
                        for m_ids_chunk in chunks(m_ids, 100):
                            # tasks for measures
                            tasks.extend(list(qs.filter(content_type__pk=measure_type.id,
                                object_id__in=m_ids_chunk).get_versionedobjects(version=version)))
                            # attentionpoints
                            ap_ids = models['attentionpoint'].objects.filter(measure__in=m_ids_chunk
                                ).get_versionedobjects(version=version
                                ).values_list('id', flat=True)
                            if ap_ids:
                                for ap_ids_chunk in chunks(ap_ids, 100):
                                    # tasks for attentionpoints
                                    tasks.extend(list(qs.filter(content_type__pk=attentionpoint_type.id,
                                        object_id__in=ap_ids_chunk).get_versionedobjects(version=version)))

    return tasks
