from django.conf import settings
from django.conf.urls import *
from django.contrib import admin
from django.views import static
from django.views.generic import TemplateView

from django_auth_policy.forms import StrictAuthenticationForm, StrictPasswordChangeForm

handler500 = 'main.views.server_error'

admin.autodiscover()

urlpatterns = patterns('',
    (r'^admin/', include(admin.site.urls)),

    (r'^versioning/', include('utils.versioning.urls')),
    (r'^hazards/', include('hazards.urls')),
    (r'^installations/', include('installations.urls')),
    (r'^analyses/', include('analyses.urls')),
    (r'^users/', include('users.urls')),

    (r'^attachments/', include('utils.attachments.urls')),
    (r'^links/', include('utils.links.urls')),
    (r'^tasks/', include('tasks.urls')),
    (r'^comments/', include('comments.urls')),
    (r'^news/', include('utils.news.urls')),
    (r'^meetings/', include('meetings.urls')),

    (r'^search/', include('utils.search.urls')),

    (r'^i18n/', include('django.conf.urls.i18n')),
    (r'^jsi18n/$', 'django.views.i18n.javascript_catalog',{'packages': ['django.conf', 'django.contrib.admin']}),
# override auth urls to use auth_policy forms
    url(r'^accounts/login/$', 'django.contrib.auth.views.login',
        name='login',
        kwargs={'authentication_form': StrictAuthenticationForm}),
    url(r'^accounts/password_change/$', 'django.contrib.auth.views.password_change',
        name='password_change',
        kwargs={'password_change_form': StrictPasswordChangeForm}),
    (r'^accounts/', include('django.contrib.auth.urls')),
    (r'^not_allowed/$', TemplateView.as_view(template_name="not_allowed.html")),

    (r'^$', TemplateView.as_view(template_name="home.html")),
    (r'^docs/markdown/$', TemplateView.as_view(template_name="docs/markdown.html")),
)

if settings.SERVE_STATIC:
    urlpatterns += patterns('',
        (r'^favicon.ico$', static.serve,{'document_root' : 'static/', 'path': 'favicon.ico'}),
        (r'^static/(?P<path>.*)$', static.serve,{'document_root' : settings.STATIC_ROOT}),
        (r'^uploads/(?P<path>.*)$', static.serve,{'document_root' : settings.MEDIA_ROOT}),
    )