# attachment views

from django.core.urlresolvers import reverse
from django.db.models.loading import get_model
from django.utils.translation import ugettext_lazy as _

from utils.attachments.models import Attachment
from utils.attachments.forms import AttachmentForm, AttachmentSheet
from utils.attachments.tables import AttachmentsTable
from utils import permissions
from utils.versioning import versionedcrud


class delete(versionedcrud.DeleteObjectView):
    model = Attachment
    form_class = AttachmentSheet

    def get_default_url(self, object):
        return display_url_for_obj(object.content_object)

class add(versionedcrud.AddObjectView):
    """
    Add an attachment to an object.
    """
    model = Attachment
    form_class = AttachmentForm
    parent_field = 'content_object'

    def __call__(self, request, app_label, model_name, *args, **kwargs):
        self.parent_class = get_model(app_label, model_name)
        return super(add, self).__call__(request, *args, **kwargs)

class display(versionedcrud.DisplayObjectView):
    allow_change = False
    allow_delete = False
    allow_report = False
    include_tasks = False
    include_comments = False
    include_history = False
    include_attachments = False
    include_links = False

    def __call__(self, request, app_label, model_name, *args, **kwargs):
        self.model = get_model(app_label, model_name)
        return super(display, self).__call__(request, *args, **kwargs)

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        attachments = Attachment.objects.attachments_for_object(object)
        table = AttachmentsTable(attachments)
        extra_context['table'] = table
        if object.may_change(self.request.user):
            extra_context['buttons'] = [
                {'url': object.get_absolute_url(),
                    'label': _('Back')},
                {'url': add_url_for_obj(object),
                    'label': _('Add')},
            ]

def add_url_for_obj(obj):
    return reverse('attachments_add', kwargs={
                        'app_label': obj._meta.app_label,
                        'model_name': obj._meta.model_name,
                        'object_id': obj.v_id
                    })

def display_url_for_obj(obj):
    return reverse('attachments_display', kwargs={
                        'app_label': obj._meta.app_label,
                        'model_name': obj._meta.model_name,
                        'object_id': obj.v_id
                    })

