"""
Tests for search app

testing with the installations fixture

"""

import datetime

from django import test
from django.conf import settings
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType

from utils import tests as utils_tests

class SearchTestCase(test.TestCase):
    fixtures = ['dummyusers', 'installations']

    def setUp(self):
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = unauthorised

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
            ('display_installations','installations','section'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

    def test_search_views(self):
        base_url = '/search/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,)

        class search(utils_tests.tablelist_TestDefinition):
            description = 'search'
            url = base_url
            users = [0,1,2,]
            rowcount = 0

        class search_filtered(utils_tests.tablelist_filtered_TestDefinition):
            description = 'filtered search'
            url = base_url
            users = [0,1,2,]
            filterdata = {"filter-query": "methanol",}
            def pretest(self, testcase, allowed):
                if testcase.user and testcase.user.id == 1:
                    self.rowcount = 1
                else:
                    self.rowcount = 0

        tests = [
            search(),
            search_filtered(),
        ]

        utils_tests.run_tests(self, users, tests)


