"""

helper functions for using versioned model objects

"""
from utils.versioning.models import VersionedForeignKey
from utils import tables

def annotate_versioned(objects, *fields):
    """
    annotate the objects with the related objects for the specified VersionedForeignKey fields

    """
    if objects:
        if not fields:
            fields = [g for g in objects[0]._meta.fields if isinstance(g, VersionedForeignKey)]
            gfk_fields = [g for g in objects[0]._meta.virtual_fields if isinstance(g, VersionedGenericForeignKey)]

        else:
            fields = [g for g in objects[0]._meta.fields if isinstance(g, VersionedForeignKey) and g.name in fields]
            gfk_fields = [g for g in objects[0]._meta.virtual_fields if isinstance(g, VersionedGenericForeignKey) and g.name in fields]

        version = getattr(objects[0], '_version', None)
        CHUNK_SIZE = 255
        if version:
            # VFK
            for field in fields:
                data_map = {}
                n = field.get_attname()
                v_id_list = [getattr(item,n) for item in objects]
                for i in xrange(0, len(v_id_list), CHUNK_SIZE):
                    v_id_list_chunk = v_id_list[i:i+CHUNK_SIZE]
                    for o in field.to._default_manager.filter(v_id__in=v_id_list_chunk).get_versionedobjects(version=version):
                        data_map[o.v_id] = o
                for item in objects:
                    val = data_map.get(getattr(item, n))
                    if val:
                        setattr(item, field.name, val)

            # VGFK
            ct_map = {}
            item_map = {}
            data_map = {}

            for item in objects:
                for gfk in gfk_fields:
                    ct_id_field = self.model._meta.get_field(gfk.ct_field).column
                    ct_map.setdefault(
                        (getattr(item, ct_id_field)), {}
                        )[getattr(item, gfk.fk_field)] = (gfk.name, item.id)
                item_map[item.id] = item

            for (ct_id), items_ in ct_map.items():
                if (ct_id):
                    ct = ContentType.objects.get_for_id(ct_id)
                    v_id_list = items_.keys()
                    for i in xrange(0, len(v_id_list), CHUNK_SIZE):
                        v_id_list_chunk = v_id_list[i:i+CHUNK_SIZE]
                        for o in ct.model_class().objects.filter(v_id__in=v_id_list_chunk).get_versionedobjects(version=version):
                            (gfk_name, item_id) = items_[o.v_id]
                            data_map[(ct_id, o.v_id)] = o

            for item in objects:
                for gfk in gfk_fields:
                    if (getattr(item, gfk.fk_field) != None):
                        ct_id_field = self.model._meta.get_field(gfk.ct_field).column
    ##                    setattr(item, gfk.name, data_map[(getattr(item, ct_id_field), getattr(item, gfk.fk_field))])
                        val = data_map.get((getattr(item, ct_id_field), getattr(item, gfk.fk_field)))
                        if val:
                            setattr(item, gfk.name, val)

    return objects


class SelectRelatedModelTable(tables.ModelTable):
    def _build_snapshot(self):
        """ Add select_related_versioned
        """
        super(SelectRelatedModelTable, self)._build_snapshot()
        self._snapshot = self._snapshot.select_related_versioned()
